#ifndef __RENDERER_H
#define __RENDERER_H
#define WIN32_MEAN_AND_LEAN
#include <windows.h>
#include <d3dx9.h>
#include "Mesh.h"

class Renderer
{
public:
	explicit Renderer(HWND wnd);
	~Renderer();

	int Get_Width() const;
	int Get_Height() const;

	void Clear();
	void Begin_Scene();
	void Set_Param(const char *param_str, float param_val);
	void Render_Mesh(const Mesh *mesh);
	void Write_Text(int x, int y, LPCTSTR str, D3DCOLOR color, bool center = false, bool big_font = false);
	void End_Scene();
	void Flip();

	void Set_World_Matrix(const D3DXMATRIX &mtx);
	void Set_View_Matrix(const D3DXMATRIX &mtx);
	void Set_Projection_Matrix();

private:
	int m_width;
	int m_height;

	LPDIRECT3D9 m_d3d;
	LPDIRECT3DDEVICE9 m_d3d_device;
	ID3DXEffect *m_effect;
	ID3DXFont *m_font_big;
	ID3DXFont *m_font_small;

	D3DXMATRIXA16 m_mtx_world;
	D3DXMATRIXA16 m_mtx_view;
	D3DXMATRIXA16 m_mtx_proj;

	void Do_Write_Text(int x, int y, LPCTSTR str, D3DCOLOR color, bool center = false, bool big_font = false);
};

#endif
