#ifndef __SINGLE_H
#define __SINGLE_H
#include <assert.h>

template <class T>
class Single
{
public:
	Single(): m_ptr(0) { }

	explicit Single(int null): m_ptr(0) { assert(null == 0); }

	explicit Single(T *ptr): m_ptr(ptr) { }

	Single(const Single &oth)
	{
		Single *o = const_cast<Single *>(&oth);
		m_ptr = o->m_ptr;
		o->m_ptr = 0;
	}

	~Single()
	{
		Release();
	}

	void Release()
	{
		if (m_ptr != 0)
			delete m_ptr;
		m_ptr = 0;
	}

	T *Get() { return m_ptr; }
	const T *Get() const { return m_ptr; }

	T &operator *() { assert(m_ptr != 0); return *m_ptr; }
	const T &operator *() const { assert(m_ptr != 0); return *m_ptr; }
	T *operator ->() { assert(m_ptr != 0); return m_ptr; }
	const T *operator ->() const { assert(m_ptr != 0); return m_ptr; }

	Single &operator =(Single &oth)
	{
		if (this == &oth)
			return *this;

		// Release old
		Release();

		// Move the ownership
		m_ptr = oth.m_ptr;
		oth.m_ptr = 0;

		return *this;
	}
	Single &operator =(int null)
	{
		assert(null == 0);
		Release();
		return *this;
	}

	bool operator ==(int null) const
	{
		assert(null == 0);
		return (m_ptr == 0);
	}
	bool operator !=(int null) const
	{
		assert(null == 0);
		return (m_ptr != 0);
	}

private:
	T *m_ptr;
};

template <class T>
Single<T> Make_Single(T *ptr)
{
	return Single<T>(ptr);
}

#endif
