#include "Math_Util.h"
#include "Tunnel.h"
#include <math.h>

Tunnel::Tunnel()
	: m_tunnel_mesh(new Mesh())
{
	Update_Mesh(0.f);
}

Tunnel::~Tunnel()
{
}

float Tunnel::Get_Radius_At(float t) const
{
	float r = 10.f - powf(t, 0.35f);
	r = Clamp(r, 0.4f, 1000.f);

	r *= 1.f + cosf(t * 20.4564451f) * 0.2f;
	return r;
}

D3DXVECTOR3 Tunnel::Get_Pos_At(float t) const
{
/*	t *= 0.01f; // Time scaling
	float nx = sinf(t + sinf(t * 1.5763f));
	float ny = sinf(t*0.1f) + cosf(t * 2.15352f) * sinf(t * 0.025465f);
	float nz = cosf(t) * cosf(t * 0.1342f);
	float radius = 50.f;*/

/*	t *= 0.2f; // Time scaling
    float p = 5.234f;// + sinf(t) * 2.f;
    float q = 11.123f;// + cosf(t) * 3.f;
    float nx = (0.5f + cosf(q * t / p)) * cosf(t);
    float ny = (0.5f + cosf(q * t / p)) * sinf(t);
    float nz = sinf(q * t / p);
	float radius = 50.f;*/

/*	t *= 0.1f; // Time scaling
    float nx = sinf(t + sinf(t * 0.5763f));
    float ny = sinf(t*0.1111f) + cosf(t * 0.85352f) * sinf(t * 0.025465f);
    float nz = cosf(t) + cosf(t * 0.1342f);
	float radius = 50.f;*/

	t *= 0.1f; // Time scaling
    float nx = sinf(t);
    float ny = cosf(t*1.1f);
    float nz = cosf(t) * sinf(t);
	float radius = 60.f;

	return D3DXVECTOR3(
		nx * radius,
		ny * radius,
		nz * radius);
}

void Tunnel::Get_Frame_At(D3DXMATRIX &dest_mtx, float t) const
{
	D3DXVECTOR3 pos1 = Get_Pos_At(t);
	D3DXVECTOR3 pos2 = Get_Pos_At(t + 0.01f);

	D3DXVECTOR3 dir = pos2 - pos1;
	D3DXVec3Normalize(&dir, &dir);

	D3DXVECTOR3 zero(0.f, 0.f, 0.f);
	D3DXVECTOR3 up(0.f, 1.f, 0.f);
	if (Abs(D3DXVec3Dot(&dir, &up)) > 0.9f)
		up = D3DXVECTOR3(1.f, 0.f, 0.f);

	D3DXMATRIXA16 tmp1, tmp2;
	D3DXMatrixTranslation(&tmp1, pos1.x, pos1.y, pos1.z);
	D3DXMatrixLookAtLH(&tmp2, &zero, &dir, &up);

	dest_mtx = tmp2 * tmp1;
}

const Mesh *Tunnel::Get_Tunnel_Mesh() const
{
	return m_tunnel_mesh.Get();
}

static D3DCOLOR Scale_Color(D3DCOLOR col, float s)
{
	DWORD a = ((col >> 24) & 0xff);
	DWORD r = ((col >> 16) & 0xff);
	DWORD g = ((col >> 8) & 0xff);
	DWORD b = ((col >> 0) & 0xff);

	return D3DCOLOR_ARGB(
		a,
		(DWORD)Clamp((float)r * s, 0.f, 255.f),
		(DWORD)Clamp((float)g * s, 0.f, 255.f),
		(DWORD)Clamp((float)b * s, 0.f, 255.f));
}

void Tunnel::Update_Mesh(float t)
{
	const float t_step = 0.5f;
	const int ring_steps = 60;
	const float ring_step = D3DX_PI * 2.f / (float)ring_steps;

	m_tunnel_mesh->Clear();

	// Normalize the positional value into an integer range
	int t_int = (int)(t / t_step);
	t = (float)t_int * t_step;

	// Setup a basic vertex
	Vertex vert;
	vert.normal = D3DXVECTOR3(0.f, 0.f, 0.f);

	D3DCOLOR color = D3DCOLOR_ARGB(255, 184, 134, 11);

	// Create the mesh for all the slots
	for (int slot = -3; slot <= 40; ++slot)
	{
		D3DXMATRIXA16 frame1;
		D3DXMATRIXA16 frame2;

		float t_val = t + t_step * (float)slot;
		Get_Frame_At(frame1, t_val);
		Get_Frame_At(frame2, t_val + t_step);

		float radius1 = Get_Radius_At(t_val);
		float radius2 = Get_Radius_At(t_val + t_step);

		D3DXVECTOR3 pos1 = Get_Mtx_Row3(frame1, 3);
		D3DXVECTOR3 dir_x1 = Get_Mtx_Col3(frame1, 0) * radius1;
		D3DXVECTOR3 dir_y1 = Get_Mtx_Col3(frame1, 1) * radius1;

		D3DXVECTOR3 pos2 = Get_Mtx_Row3(frame2, 3);
		D3DXVECTOR3 dir_x2 = Get_Mtx_Col3(frame2, 0) * radius2;
		D3DXVECTOR3 dir_y2 = Get_Mtx_Col3(frame2, 1) * radius2;

		// DEBUG: Set a test color
/*		switch (Abs((t_int + slot) % 4))
		{
		case 0:		vert.color = D3DCOLOR_ARGB(255, 255, 255, 255); break;
		case 1:		vert.color = D3DCOLOR_ARGB(255, 255, 0, 0); break;
		case 2:		vert.color = D3DCOLOR_ARGB(255, 0, 255, 0); break;
		case 3:		vert.color = D3DCOLOR_ARGB(255, 0, 0, 255); break;
		}*/

		// Generate one ring of the tunnel
		bool start_strip = true;
		for (int i = 0; i <= ring_steps; ++i)
		{
			float ang1 = (float)i * ring_step;
			float ang2 = ang1 + ring_step;

			D3DXVECTOR3 p11 = cosf(ang1) * dir_x1 + sinf(ang1) * dir_y1;
			D3DXVECTOR3 p21 = cosf(ang1) * dir_x2 + sinf(ang1) * dir_y2;
			D3DXVECTOR3 p12 = cosf(ang2) * dir_x1 + sinf(ang2) * dir_y1;
			D3DXVECTOR3 p22 = cosf(ang2) * dir_x2 + sinf(ang2) * dir_y2;

			float col_scale1 = 1.f + sinf(ang1 * 15.f) * 0.1f;
			float col_scale2 = 1.f + sinf(ang2 * 15.f) * 0.1f;

			vert.pos = pos1 + p11;
			vert.normal = -p11;

			D3DXVec3Normalize(&vert.normal, &vert.normal);
			vert.color = Scale_Color(color, col_scale1);
			m_tunnel_mesh->Add_Vertex(vert, start_strip);

			vert.pos = pos2 + p21;
			vert.normal = -p21;
			D3DXVec3Normalize(&vert.normal, &vert.normal);
			vert.color = Scale_Color(color, col_scale1);
			m_tunnel_mesh->Add_Vertex(vert);

			vert.pos = pos1 + p12;
			vert.normal = -p12;
			D3DXVec3Normalize(&vert.normal, &vert.normal);
			vert.color = Scale_Color(color, col_scale2);
			m_tunnel_mesh->Add_Vertex(vert);

			vert.pos = pos2 + p22;
			vert.normal = -p22;
			D3DXVec3Normalize(&vert.normal, &vert.normal);
			vert.color = Scale_Color(color, col_scale2);
			m_tunnel_mesh->Add_Vertex(vert);

			start_strip = false;
		}
	}
}
